/* expoo.c
 * Small Programm to generate exponential on/off traffic
 *
 * Copyright 2004, Holger Fuessler <fuessler@informatik.uni-mannheim.de>
 *
 * $Id: expoo.c,v 1.1 2004/05/11 08:26:15 fuessler Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <math.h>


#define MEAN_BURST	1.004	
#define MEAN_IDLE		1.587
#define RATE	88000



double get_expo(GRand * gprng, double lambda)
{
	return(-1.0 * lambda *  
						log(g_rand_double(gprng))
					);
} /* EO get_expo() */

int main (int argc, char** argv)
{
	if ((argc != 2) && (atof(argv[1]) > 0)){
		fprintf(stderr, "Call with expoo <MAX_TIME>\n");
		exit(1);
	}
{
	double now, max_time, next_event;
	double sum_burst, sum_idle;
	long no_burst, no_idle;
	int send, first;
	GRand * gprng;
	now = 0.0;
	send = 0;
	first = 0;
	gprng = g_rand_new();
	max_time = atof(argv[1]);
	sum_burst = sum_idle = 0.0;
	no_burst = no_idle = 0;

	while (now < max_time){
		if (! first){
			printf("%-9.6f\t%d\n", now, (send ? 0 : RATE));
		}
		first = 0;
		printf("%-9.6f\t%d\n", now, (send ? RATE : 0));
		if (send) {
			next_event = get_expo(gprng, MEAN_BURST);
			printf("# Burst_Length: %f\n", next_event);
			sum_burst += next_event;
			no_burst++;
			send = 0;
		}else{
			next_event = get_expo(gprng, MEAN_IDLE);
			printf("# Idle_Length: %f\n", next_event);
			sum_idle += next_event;
			no_idle++;
			send = 1;
		}
		now += next_event;
	}

	if (send){
		printf("%-9.6f\t%d\n", 50.0,  0);
	}
	printf("# Average Burst (Setting): %f Average Idle (Setting): %f Rate (Setting): %d\n", MEAN_BURST, MEAN_IDLE, RATE);	
	printf("# Average Burst  (Actual): %f Average Idle  (Actual): %f\n", (sum_burst / no_burst), (sum_idle / no_idle));	
	g_rand_free(gprng);
	}
	exit(0);
} /* EO main */
