/* randspeed.c
 * Small Programm to test speed of different ways of obtaining discrete random variables
 *
 * Copyright 2004, Holger Fuessler <fuessler@informatik.uni-mannheim.de>
 *
 * $Id: randspeed.c,v 1.2 2004/05/06 15:01:08 fuessler Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>

#define NO_OF_DRAWS 100000000
#define MAX_VAL 23 
#define BINARY_OP 0xf

int getDVBinaryAnd(GRand * prng){
	guint32 val;
	val = g_rand_int(prng);
	return(val & BINARY_OP);
}

int getDVInt(GRand * prng){
	guint32 val;
	val = g_rand_int(prng);
	return(val % MAX_VAL);
}

int getDVFloat(GRand * prng){
	double val;
	val = g_rand_double(prng);
	return( (int) (val * MAX_VAL) );
}

void run(int number){
	
	if (number == 1){
		GRand * gprng;
		long i;
		int value;
		printf("Initializing grand() Mersenne Twister\n");
		gprng = g_rand_new();
		printf("Drawing %d times...\n", NO_OF_DRAWS);
		for (i = 0; i < NO_OF_DRAWS; i++){
			value = getDVInt(gprng);
		}
		g_rand_free(gprng);
	}

	if (number == 2){
		GRand * gprng;
		long i;
		int value;
		printf("Initializing grand() Mersenne Twister\n");
		gprng = g_rand_new();
		printf("Drawing %d times...\n", NO_OF_DRAWS);
		for (i = 0; i < NO_OF_DRAWS; i++){
			value = getDVFloat(gprng);
		}
		g_rand_free(gprng);
	}

	if (number == 3){
		GRand * gprng;
		long i;
		int value;
		printf("Initializing grand() Mersenne Twister\n");
		gprng = g_rand_new();
		printf("Drawing %d times...\n", NO_OF_DRAWS);
		for (i = 0; i < NO_OF_DRAWS; i++){
			value = getDVBinaryAnd(gprng);
		}
		g_rand_free(gprng);
	}

} /* EO run() */

int main (int argc, char** argv)
{
	if (argc != 2){
		fprintf(stderr, "Call with randspeed2 <METHOD>\n, with METHOD=1 (int) or METHOD=2 (float) or METHOD=3 (binary and)\n");
		exit(1);
	}

	run(atoi(argv[1]));
	
	printf("Done!\n");
	exit(-1);
} /* EO main */
